static bool action_is_legal(Game *game, Action *act, Action *prev_actions, u32 nprev_actions) {
	switch (act->kind) {
	case ACT_MOVE:
	case ACT_ATTACK: {
		HexPos h1 = act->hex1, h2 = act->hex2;
		Action *a, *end;
		if (!(hex_can_have_pion(game, h1) && hex_can_have_pion(game, h2)
			&& hexes_are_adjacent(h1, h2))) {
			return false;
		}
		if (act->kind == ACT_MOVE) {
			// check if there's any other move action with the same hex1
			for (a = prev_actions, end = a + nprev_actions; a != end; ++a) {
				if (a->kind == ACT_MOVE) {
					if (hex_eq(a->hex1, h1))
						return false;
				}
			}
		}
		return true;
	}
	case ACT_NONE: break;
	}
	return false;
}

// what is the cost of this action for the current player? 
static u32 action_cost(Game *game, Action *action) {
	u32 cost = U32_MAX;
	(void)game;
	switch (action->kind) {
	case ACT_NONE: break;
	case ACT_MOVE:
		cost = BASIC_ATTACK_POINT_COST;
		break;
	case ACT_ATTACK:
		cost = BASIC_ATTACK_POINT_COST;
		break;
	}
	return cost;
}

static bool action_eq(Action const *a1, Action const *a2) {
	if (a1->kind != a2->kind) return false;
	switch (a1->kind) {
	case ACT_NONE: return true;
	case ACT_MOVE:
	case ACT_ATTACK:
		return hex_eq(a1->hex1, a2->hex1) && hex_eq(a1->hex2, a2->hex2);
	}
	return false;
}

static void actions_remove_all_at_hex(Game *game, HexPos pos) {
	uint player = game->controlled_player;
	Action *action, *end;
	for (action = game->actions[player], end = action + game->nactions[player]; action != end; ++action) {
		if (hex_eq(action->hex1, pos) || hex_eq(action->hex2, pos)) {
			game->points_left += action_cost(game, action);
			if (--game->nactions[player])
				*action-- = *--end;
		}
	}
}
					
static void not_enough_points(State *state) {					
	set_big_fading_message(state, error_message_color, "Out of points.");
}

// finish game->current_action 
static void action_finish_current(State *state) {
	Game *game = &state->game;
	Action *curr_action = &game->current_action;
	uint player = game->controlled_player;
	u32 cost = action_cost(game, curr_action);

	if (action_is_legal(game, curr_action, game->actions[player], game->nactions[player])) {
		if (game->nactions[player] < ACTIONS_MAX) {
			Action *act, *act_end;
			bool done_before = false;
			// check if we've done exactly this action before 
			for (act = game->actions[player], act_end = game->actions[player]+game->nactions[player];
				act != act_end; ++act) {
				if (action_eq(act, curr_action)) {
					done_before = true;
					break;
				}
			}
			if (!done_before) {
				if (game->points_left >= cost) {
					game->points_left -= cost;
					act = &game->actions[player][game->nactions[player]++];
					*act = *curr_action;
				} else {
					not_enough_points(state);
				}
			}
		} else {
			set_big_fading_message(state, error_message_color, "Reached maximum actions (" STRINGIFY(ACTIONS_MAX) ").");
		}
	}
	
	curr_action->kind = ACT_NONE;
}

