#define SETTINGS_FILENAME "settings.txt"

static bool settings_write(State *state) {
	Settings *settings = &state->settings;
	FILE *fp = fopen(SETTINGS_FILENAME, "w");
	if (fp) {
		fprintf(fp, "fullscreen: %d\n", settings->fullscreen);
		fprintf(fp, "label pions: %d\n", settings->label_pions);
		fprintf(fp, "text size: %d\n", settings->text_size);
		fprintf(fp, "previews: %d\n", settings->show_map_previews);
		fclose(fp);
	} else {
		printf("!!! Failed to write settings.\n");
		return false;
	}
	return true;
}

static void settings_default(Settings *settings) {
	settings->fullscreen = true;
	settings->label_pions = false;
	settings->text_size = 0;
	settings->show_map_previews = true;
}

// read settings, or use default settings if settings file does not exist 
static bool settings_read(State *state) {
	Settings *settings = &state->settings;
	FILE *fp = fopen(SETTINGS_FILENAME, "r");
	if (fp) {
		char line[256] = {0};
		settings_default(settings); // start out with default settings (in case the settings file is missing something) 
		while (fgets(line, sizeof line, fp)) {
			char *colon = strstr(line, ": ");
			char *key, *value;
			if (line[0] == '#' || !colon) continue;
			key = line;
			value = colon + 2;
			*colon = 0;
			if (streq(key, "fullscreen")) {
				settings->fullscreen = (bool)atoi(value);
			} else if (streq(key, "label pions")) {
				settings->label_pions = (bool)atoi(value);
			} else if (streq(key, "text size")) {
				settings->text_size = (u8)atoi(value);
			} else if (streq(key, "previews")) {
				settings->show_map_previews = (bool)atoi(value);
			} else {
				debug_log("!! Warning: property %s not recognized.\n", key);
			}
		}
	} else {
		settings_default(settings);
		settings_write(state); // write out the default settings 
		return false;
	}
	return true;
}

