// gets the state from the frame
static State *frame_state(Frame *frame) {
	return (State *)frame->memory;
}

static v2 pixels_to_gl_coords(State *state, int x, int y) {
	return V2((float)x / (float)state->win_height,
		(float)y / (float)state->win_height);
}

// one pixel in openGL coordinates 
static float gl_pixel(State *state) {
	return 1 / (float)state->win_height;
}

// allocates aligned temporary memory 
static u8 *tmp_alloc(State *state, size_t bytes) {
	u32 used = state->tmp_mem_used;
	u32 max_aligns_needed = (u32)(bytes + sizeof(MaxAlign) - 1) / (u32)sizeof(MaxAlign); // = ceil(bytes / sizeof(MaxAlign)) 
	MaxAlign *ret = state->tmp_mem + used;
	if (bytes == 0) {
		return NULL;
	}
	if (used + max_aligns_needed > arr_count(state->tmp_mem)) {
		assert(0);
		return NULL;
	}
	state->tmp_mem_used += max_aligns_needed;
	memset(ret, 0, bytes);
	return (u8 *)ret;
}

#define tmp_alloc_object(state, type) ((type *)tmp_alloc((state), sizeof(type)))
#define tmp_alloc_arr(state, type, n) ((type *)tmp_alloc((state), (n) * sizeof(type)))

/* 
	these functions save and restore the state of the temporary memory.
*/

static u32 tmp_push(State *state) {
	return state->tmp_mem_used;
}

static void tmp_pop(State *state, u32 mark) {
	state->tmp_mem_used = mark;
}

static void set_big_fading_message(State *state, v4 color, char const *message, ...) {
	va_list args;
	va_start(args, message);
	vsprintf(state->big_fading_message, message, args);
	va_end(args);
	state->big_fading_message_time = 0;
	state->big_fading_message_color = color;
}

static void display_error(State *state, GameScreen return_to_screen, bool is_bug, char const *fmt, ...) {
	va_list args;
	va_start(args, fmt);
#if _POSIX_C_SOURCE >= 200112L
	vsnprintf(state->error_message, sizeof state->error_message, fmt, args);
#else
	vsprintf(state->error_message, fmt, args);
#endif
	state->is_error_bug = is_bug;
	state->return_to_screen = return_to_screen;
	va_end(args);
	state->screen = SCR_ERROR;
}

static bool mouse_in_rect(State *state, Rect r) {
	return rect_contains_point(r, state->mouse_pos);
}
