#define OLD_VERSION_NUMBER 1

#include <stdlib.h>
#include <inttypes.h>
#include <string.h>
#include <stdio.h>
#include <dirent.h>
#include <assert.h>

typedef uint8_t  u8;
typedef uint16_t u16;
typedef uint32_t u32;

static void insert_bytes(u8 *data, size_t *size, u8 *bytes, size_t nbytes, size_t pos) {
	size_t old_size = *size;
	size_t new_size = old_size + nbytes;
	*size = new_size;

	memmove(data + pos + nbytes, data + pos, old_size - pos);
	memcpy(data + pos, bytes, nbytes);
}

static void insert_u32(u8 *data, size_t *size, u32 x, size_t pos) {
	insert_bytes(data, size, (u8 *)&x, sizeof x, pos);
}

static void change_u32(u8 *data, u32 new_u32, size_t pos) {
	memcpy(data + pos, &new_u32, sizeof new_u32);
}

static size_t update_version(u8 *file_data, size_t size) {
	u16 width, height;
	u32 area;
	u8 *zeros;
	memcpy(&width,  file_data + 6, sizeof width);
	memcpy(&height, file_data + 8, sizeof height);

	area = (u32)width * height;
	zeros = calloc(area, 1);
	assert(zeros);
	insert_bytes(file_data, &size, zeros, area, 14 + area);


	change_u32(file_data, 2, 0);
	insert_u32(file_data, &size, 2, 0);


	free(zeros);
	return size;
}

int main(void) {
	DIR *dir = opendir("starting_maps");
	int ret = 0;
#define MAX_FILE_SIZE 1000000
	static u8 file_data[MAX_FILE_SIZE*2];

	if (!dir) {
		fprintf(stderr, "Couldn't open starting_maps.");
		ret = EXIT_FAILURE;
	} else {
		if (system("mkdir -p /tmp/starting_maps_backup && cp -r starting_maps/* /tmp/starting_maps_backup/")) {
			fprintf(stderr, "Error backing up starting maps.\n");
			ret = EXIT_FAILURE;
		} else {
			struct dirent *ent;
			while ((ent = readdir(dir))) {
				if (ent->d_type == DT_REG) {
					char path_in[256], path_out[256];
					FILE *in, *out;
					snprintf(path_in, sizeof path_in, "/tmp/starting_maps_backup/%s", ent->d_name);
					snprintf(path_out, sizeof path_out, "starting_maps/%s", ent->d_name);
					
					in = fopen(path_in, "rb");
					if (!in) {
						fprintf(stderr, "Error opening %s.\n", path_in);
						ret = EXIT_FAILURE;
					} else {
						u32 version_number = 0;
						fread(&version_number, sizeof version_number, 1, in);
						if (version_number != OLD_VERSION_NUMBER) {
							printf("Warning: Expected version %d in file %s, got %d.\n", OLD_VERSION_NUMBER, ent->d_name, version_number);
						} else {
							size_t file_size;

							fseek(in, 0, SEEK_END);
							file_size = (size_t)ftell(in);
							fseek(in, 0, SEEK_SET);
							if (file_size > MAX_FILE_SIZE) {
								fprintf(stderr, "File too large (size: %ld).\n", (long)file_size);
								ret = EXIT_FAILURE;
							} else {
								if (fread(file_data, 1, file_size, in) != file_size) {
									fprintf(stderr, "File read error (file: %s).\n", ent->d_name);
									ret = EXIT_FAILURE;
								} else {
									out = fopen(path_out, "wb");
									if (!out) {
										fprintf(stderr, "Couldn't open output file %s.\n", path_out);
										ret = EXIT_FAILURE;
									} else {
										file_size = update_version(file_data, file_size);

										if (fwrite(file_data, 1, file_size, out) != file_size) {
											fprintf(stderr, "Error writing to %s.\n", path_out);
											ret = EXIT_FAILURE;
										}

										fclose(out);
									}
								}
							}
						}
						fclose(in);
					}

					if (ret == EXIT_FAILURE) break;
				}
			}
		}
		closedir(dir);
	}
	return ret;
}
